#include "frame.h"
#include <QPainter>
#include <cmath>
#define sgn(a) ( (a) < 0 )
using namespace std;

extern bool stepbystep;


Frame::Frame(QWidget *parent)
    : QWidget(parent)
{
    setBackgroundRole(QPalette::Base);
    setAutoFillBackground(true);
    echx=0.0;
    echy=0.0;
}
QSize Frame::minimumSizeHint() const
{
    return QSize(500, 500);
}
QSize Frame::sizeHint() const
{
    return QSize(500, 500);
}
void Frame::setPriorDomain(Box &b)
{
    priorDomain = b;
    echx = width()/(priorDomain[1].width());
    echy = height()/(priorDomain[2].width());
}
double Frame::xToPix(double x)
{
    return (x-priorDomain[1].getInf())*echx;
}
double Frame::yToPix(double y)
{
    return height()-(y-priorDomain[2].getInf())*echy;
}
void Frame::clear()
{
    l.clear();
    update();
}
void Frame::addBox(Box &b, QColor c)
{
    QPoint topleft;
    QPoint bottomright;
    coloredRect cr;
    Interval ix=b[1];
    Interval iy=b[2];
    topleft.setX(xToPix(ix.getInf()));
    topleft.setY(yToPix(iy.getSup()));
    bottomright.setX(xToPix(ix.getSup()));
    bottomright.setY(yToPix(iy.getInf()));
    cr.rect.setTopLeft(topleft);
    cr.rect.setBottomRight(bottomright);
    cr.color=c;
    l.push_front(cr);
    if (stepbystep)
        repaint();
    update();
}

void Frame::paintEvent(QPaintEvent * /* event */)
{
    QPainter painter(this);
    QList<coloredRect>::iterator li;
    for(li=this->l.begin(); li != this->l.end(); ++li)
    {
        painter.fillRect((*li).rect,(*li).color );
        painter.drawRect((*li).rect);
    }
    painter.setBrush(QBrush(Qt::NoBrush));
    painter.setPen(QPen(QColor(Qt::red)));
    painter.drawRect(QRectF(0, 0, width() - 1, height() - 1));
}
